package com.next14.proximity_flutter;
import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.util.Log;
import android.widget.Toast;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.jointag.proximity.ProximitySDK;
import com.jointag.proximity.cmp.ManualConsent;
import com.jointag.proximity.scheduler.Scheduler;
import com.jointag.proximity.util.Logger;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Dictionary;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.concurrent.Callable;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import kotlin.reflect.KFunction;

/** FlutterJointagPlugin */
public class FlutterProximityPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {



  private MethodChannel channel;
  //private EventChannel eventChannel;
  private EventChannel eventChannel;
  private Context context;
  private static Activity activity;


  public static void Initialize(Application application, String apiKey, String apiSecret ) {
    try {
      // Set a custom logger tag
      Logger.setTag("FlutterJointag");
      // Enable verbose logging
      Logger.setLogLevel(Logger.VERBOSE);
      ProximitySDK.init(application, apiKey, apiSecret);
    }
    catch(Exception e) {
      Log.d("Error", e.getMessage());
    }
  }

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    context = flutterPluginBinding.getApplicationContext();
    channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "jtproximity_flutter");
    channel.setMethodCallHandler(this);

    eventChannel = new EventChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor().getBinaryMessenger(), "events");
    eventChannel.setStreamHandler(new StreamHandler());
    /*
    // Prepare channel
    eventChannel = new EventChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor().getBinaryMessenger(), "events");
    eventChannel.setStreamHandler(new EventChannel.StreamHandler() {
      @Override
      public void onListen(Object listener, EventChannel.EventSink eventSink) {
        startListening(listener, eventSink);
      }

      @Override
      public void onCancel(Object listener) {
        cancelListening(listener);
      }
    });
    */
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "jtproximity_flutter");
    channel.setMethodCallHandler(new FlutterProximityPlugin());
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    switch (call.method) {
      case "setExternalUserId":
        setExternalUserId(call, result);
        break;
      case "sendTag":
        sendTag(call, result);
        break;
      case "sendTags":
        sendTags(call, result);
        break;
      case "getProfilingConsent":
        getProfilingConsent(result);
        break;
      case "getMonitoringConsent":
        getMonitoringConsent(result);
        break;
      case "getAdvertisingConsent":
        getAdvertisingConsent(result);
        break;
      case "getAdvancedConsent":
        getAdvancedConsent(result);
        break;
      case "enableAdvancedConsent":
        enableAdvancedConsent(result);
        break;
      case "enableProfilingConsent":
        enableProfilingConsent(result);
        break;
      case "enableMonitoringConsent":
        enableMonitoringConsent(result);
        break;
      case "enableAdvertisingConsent":
        enableAdvertisingConsent(result);
        break;
      case "disableProfilingConsent":
        disableProfilingConsent(result);
        break;
      case "disableMonitoringConsent":
        disableMonitoringConsent(result);
        break;
      case "disableAdvertisingConsent":
        disableAdvertisingConsent(result);
        break;
      case "disableAdvancedConsent":
        disableAdvancedConsent(result);
        break;
      case "checkPendingPermissions":
        checkPendingPermissions(result);
        break;
      case "getInstallationId":
        getInstallationId(result);
        break;
      case "setAdvertisingEnabled":
        setAdvertisingEnabled(result, true);
        break;
      case "setAdvertisingDisabled":
        setAdvertisingEnabled(result, false);
        break;
      case "enabledCmp":
        enabledCmp(result);
        break;
      case "setBaseJobId":
        setBaseJobId(call, result);
        break;
      case "getAdvertisingIdentifier":
        getAdvertisingIdentifier(result);
        break;
      case "getApiKey":
        getApiKey(result);
        break;
      case "getContext":
        getContext(result);
        break;
      case "getGDPRConsent":
        getGDPRConsent(call, result);
        break;
      case "getSecret":
        getSecret(result);
        break;
      case "getSessionId":
        getSessionId(result);
        break;
      case "isLimitAdTrackingEnabled":
        isLimitAdTrackingEnabled(result);
        break;
        //da fare
      case "getCustomActionListeners":
        getCustomActionListeners(result);
        break;
      case "addCustomActionListener":
        ProximitySDK.getInstance().addCustomActionListener(payload -> Toast.makeText(context, "Received custom action " + payload, Toast.LENGTH_LONG).show());
        break;
      case "removeCustomActionListener":
        ProximitySDK.getInstance().removeCustomActionListener(null);
        break;

      case "getAdvertisingEnabled":
        getAdvertisingEnabled(result);
        break;
      case "setGDPRConsent":
        setGDPRConsent(call, result);
        break;
      default:
        result.notImplemented();
        break;
    }
  }

  private void getProfilingConsent(@NonNull Result result) {
    try {
      result.success(ProximitySDK.getInstance().getManualConsent(ManualConsent.Profiling));
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }
  private void getMonitoringConsent(@NonNull Result result) {
    try {
      result.success(ProximitySDK.getInstance().getManualConsent(ManualConsent.Monitoring));
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }
  private void getAdvertisingConsent(@NonNull Result result) {
    try {
      result.success(ProximitySDK.getInstance().getManualConsent(ManualConsent.Advertising));
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }
  private void getAdvancedConsent(@NonNull Result result) {
    try {
      result.success(ProximitySDK.getInstance().getManualConsent(ManualConsent.AdvancedTracking));
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }

  private void enableAdvancedConsent(@NonNull Result result) {
    try {
      ProximitySDK.getInstance().setManualConsent(ManualConsent.AdvancedTracking, true);
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
    return;
  }
  private void enableProfilingConsent(@NonNull Result result) {
    try {
      ProximitySDK.getInstance().setManualConsent(ManualConsent.Profiling, true);
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
    return;
  }
  private void enableMonitoringConsent(@NonNull Result result) {
    try {
      ProximitySDK.getInstance().setManualConsent(ManualConsent.Monitoring, true);
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
    return;
  }
  private void enableAdvertisingConsent(@NonNull Result result) {
    try {
      ProximitySDK.getInstance().setManualConsent(ManualConsent.Advertising, true);
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
    return;
  }
  private void disableProfilingConsent(@NonNull Result result) {
    try {
      ProximitySDK.getInstance().setManualConsent(ManualConsent.Profiling, false);
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
    return;
  }
  private void disableMonitoringConsent(@NonNull Result result) {
    try {
      ProximitySDK.getInstance().setManualConsent(ManualConsent.Monitoring, false);
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
    return;
  }
  private void disableAdvertisingConsent(@NonNull Result result) {
    try {
      ProximitySDK.getInstance().setManualConsent(ManualConsent.Advertising, false);
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
    return;
  }
  private void disableAdvancedConsent(@NonNull Result result) {
    try {
      ProximitySDK.getInstance().setManualConsent(ManualConsent.AdvancedTracking, false);
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
    return;
  }

  private void getCustomActionListeners(@NonNull Result result) {
    try {

      result.success(ProximitySDK.getInstance().getCustomActionListeners());
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }
  private void getAdvertisingEnabled(@NonNull Result result) {
    try {
      result.success(ProximitySDK.getInstance().getAdvertisingEnabled());
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }
  private void setGDPRConsent(@NonNull MethodCall call, @NonNull Result result) {
    try {
      ProximitySDK.getInstance().setGDPRConsent(call.argument("vendor"),call.argument("consent"));
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
    return;
  }
  private void setExternalUserId(@NonNull MethodCall call, @NonNull Result result) {
    try {
      ProximitySDK.getInstance().setExternalUserId(call.argument("userId"));
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
    return;
  }

  private void sendTag(@NonNull MethodCall call, @NonNull Result result) {
    try {
      ProximitySDK.getInstance().sendTag(call.argument("key"),call.argument("value"));
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
    return;
  }

  private void sendTags(@NonNull MethodCall call, @NonNull Result result) {
    try {
      ProximitySDK.getInstance().sendTags(call.argument("tags"));
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
    return;
  }

  private void isLimitAdTrackingEnabled(@NonNull Result result) {
    try {
      result.success(ProximitySDK.getInstance().isLimitAdTrackingEnabled());
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }

  private void getSessionId(@NonNull Result result) {
    try {
      result.success(ProximitySDK.getInstance().getSessionId());
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }

  private void getSecret(@NonNull Result result) {
    try {
      result.success(ProximitySDK.getInstance().getSecret());
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }

  private void getGDPRConsent(@NonNull MethodCall call, @NonNull Result result) {
    try {
      result.success(ProximitySDK.getInstance().getGDPRConsent(call.argument("VENDOR_ID")));
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }

  private void getContext(@NonNull Result result) {
    try {
      result.success(ProximitySDK.getInstance().getContext());
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }

  private void getApiKey(@NonNull Result result) {
    try {
      result.success(ProximitySDK.getInstance().getApiKey());
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }

  private void getAdvertisingIdentifier(@NonNull Result result) {
    try {
      result.success(ProximitySDK.getInstance().getAdvertisingIdentifier());
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }

  private void setBaseJobId(@NonNull MethodCall call, @NonNull Result result) {
    try {
      Scheduler.setBaseJobId(call.argument("newJobID"));
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }

  private void enabledCmp(@NonNull Result result) {
    try {
      ProximitySDK.enabledCmp();
      result.success(true);
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }

  private void setAdvertisingEnabled(@NonNull Result result, boolean b) {
    try {
      ProximitySDK.getInstance().setAdvertisingEnabled(b);
      result.success(ProximitySDK.getInstance().getAdvertisingEnabled());
    } catch (Exception e) {
      result.error("Generic Exception",e.getMessage(),null);
    }
  }

  private void getInstallationId(@NonNull Result result) {
    try {
      result.success(ProximitySDK.getInstance().getInstallationId());
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }

  private void checkPendingPermissions(@NonNull Result result) {
    try {
      ProximitySDK.getInstance().checkPendingPermissions();
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }

  private void Init(@NonNull MethodCall call, @NonNull Result result) {
    try {
      // Set a custom logger tag
      Logger.setTag("FlutterJointag");
      // Enable verbose logging
      Logger.setLogLevel(Logger.VERBOSE);
      ProximitySDK.init(activity.getApplication(), call.argument("API_KEY"), call.argument("SECRET"));
    }
    catch(Exception e) {
      result.error("Generic Exception",e.getMessage(), null);
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
    activity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

  }

  @Override
  public void onDetachedFromActivity() {

  }

  // Listeners
  private Map<Object, Runnable> listeners = new HashMap<>();

  void OLDstartListening(Object listener, EventChannel.EventSink emitter) {
    // Prepare a timer like self calling task
    final Handler handler = new Handler();

    ProximitySDK.getInstance().addCustomActionListener(payload -> {
      new Runnable() {
        @Override
        public void run() {
          // Send some value to callback
          emitter.success(payload);
        }
      };
    });

    // Run task
    handler.postDelayed(listeners.get(listener), 1000);
  }
  void startListening(Object listener, EventChannel.EventSink emitter) {
    // Prepare a timer like self calling task
    final Handler handler = new Handler();
    listeners.put(listener, new Runnable() {
      @Override
      public void run() {
        //if (listeners.containsKey(listener)) {
          ProximitySDK.getInstance().addCustomActionListener(payload -> emitter.success(payload));
          // Send some value to callback
          //emitter.success("Hello listener: " + listener.toString() + " at:" + (System.currentTimeMillis() / 1000));
          //handler.postDelayed(this, 1000);
        //}
      }
    });
    // Run task
    handler.postDelayed(listeners.get(listener), 1000);
  }
  void cancelListening(Object listener) {
    // Remove callback
    if (listener != null) {
      listeners.remove(listener);
      Log.d("Cancelling", "Listener: " + listeners.toString());
    }
  }


  public static class StreamHandler implements EventChannel.StreamHandler {
    private final Handler handler = new Handler();
    private final Runnable runnable = new Runnable() {
      @Override
      public void run() {
        ProximitySDK.getInstance().addCustomActionListener(payload -> {
          Intent intent = new Intent(activity, activity.getClass());
          intent.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
          activity.startActivity(intent);
          eventSink.success(payload);
        });
      }
    };

    private EventChannel.EventSink eventSink;

    @Override
    public void onListen(Object o, final EventChannel.EventSink eventSink) {
      this.eventSink = eventSink;
      runnable.run();
    }

    @Override
    public void onCancel(Object o) {
      //ProximitySDK.getInstance().removeCustomActionListener(sdkRunnables.get(runnable));

      handler.removeCallbacks(runnable);
    }
  }


}


