import Flutter
import UIKit
import JTProximitySDK

public class SwiftJTProximityFlutterPlugin: NSObject, FlutterPlugin {
    public static func initializeWithOptions(launchOptions: [UIApplication.LaunchOptionsKey: Any]?, apiKey: String, apiSecret: String) {
        ProximitySDK.shared.setLogLevel(JTPLogLevel.verbose)
        ProximitySDK.shared.initWithLaunchOptions(launchOptions, apiKey: apiKey, apiSecret: apiSecret)
    }
    
    init(_ registrar: FlutterPluginRegistrar)
    {
        super.init()
        let eventChannel = FlutterEventChannel.init(name: "events", binaryMessenger: registrar.messenger());
        let handler = SwiftStreamHandler()
        eventChannel.setStreamHandler(handler)
        ProximitySDK.shared.customDelegate = handler
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "jtproximity_flutter", binaryMessenger: registrar.messenger())

        let instance = SwiftJTProximityFlutterPlugin(registrar)
        registrar.addMethodCallDelegate(instance, channel: channel)
        registrar.addApplicationDelegate(instance)
    }

    public func applicationDidBecomeActive(_ application: UIApplication) {
        debugPrint("applicationDidBecomeActive")
    }

    public func applicationWillTerminate(_ application: UIApplication) {
        debugPrint("applicationWillTerminate")
    }

    public func applicationWillResignActive(_ application: UIApplication) {
        debugPrint("applicationWillResignActive")
    }

    public func applicationDidEnterBackground(_ application: UIApplication) {
        debugPrint("applicationDidEnterBackground")
    }

    public func applicationWillEnterForeground(_ application: UIApplication) {
        print("applicationWillEnterForeground")
    }
    
    public func application(_ application: UIApplication, didReceive notification: UILocalNotification) {
        debugPrint("notification")
        
        if #available(iOS 10.0, *) {
            if (ProximitySDK.shared.application(application, didReceive: notification)){
                return;
            }
        } else {
            // Fallback on earlier versions
        }
    }
    @available(iOS 10.0, *)
    public func userNotificationCenter(_ center: UNUserNotificationCenter, didReceive response: UNNotificationResponse, withCompletionHandler completionHandler: @escaping () -> Void) {
        if ProximitySDK.shared.userNotificationCenter(center, didReceive: response) {
            //qui arriva il tap di tutte le notifiche, anche di quelle custom...
            completionHandler()
            return
        }
    }
    @available(iOS 10.0, *)
    public func userNotificationCenter(_ center: UNUserNotificationCenter, willPresent notification: UNNotification, withCompletionHandler completionHandler: @escaping (UNNotificationPresentationOptions) -> Void) {
        
        if ProximitySDK.shared.userNotificationCenter(center, willPresent: notification) {
            completionHandler([.alert, .badge, .sound])
            return
        }
    }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "setExternalUserId":
            guard let args = call.arguments else {
                return
            }
            if let myArgs = args as? [String: Any],
                let userId = myArgs["userId"] as? String{
                ProximitySDK.shared.externalUserId = userId;
                result(true);
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                   "flutter arguments in method: (getGDPRConsent)", details: nil))
            }
            break;
        case "sendTag":
            guard let args = call.arguments else {
                return
            }
            if let myArgs = args as? [String: Any],
                let key = myArgs["key"] as? String,
                let value = myArgs["value"]{
                ProximitySDK.shared.sendTag(value, for: key);
                result(true);
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                   "flutter arguments in method: (sendTag)", details: nil))
            }
            break;
        case "sendTags":
            guard let args = call.arguments else {
                return
            }
            if let myArgs = args as? [String: Any],
               let tags = myArgs["tags"] as? [String : Any]{
                ProximitySDK.shared.sendTags(tags);
                result(true);
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                   "flutter arguments in method: (sendTag)", details: nil))
            }
            break;
        case "getProfilingConsent":
            result(ProximitySDK.shared.getManualConsent(for: .profiling));
            break;
        case "getMonitoringConsent":
            result(ProximitySDK.shared.getManualConsent(for: .monitoring));
            break;
        case "getAdvertisingConsent":
            result(ProximitySDK.shared.getManualConsent(for: .advertising));
            break;
        case "getAdvancedConsent":
            result(ProximitySDK.shared.getManualConsent(for: .advancedTracking));
            break;
        case "enableAdvancedConsent":
            ProximitySDK.shared.setManualConsent(true, for: .advancedTracking);
            result(true);
            break;
        case "enableProfilingConsent":
            ProximitySDK.shared.setManualConsent(true, for: .profiling);
            result(true);
            break;
        case "enableMonitoringConsent":
            ProximitySDK.shared.setManualConsent(true, for: .monitoring);
            result(true);
            break;
        case "enableAdvertisingConsent":
            ProximitySDK.shared.setManualConsent(true, for: .advertising);
            result(true);
            break;
        case "disableProfilingConsent":
            ProximitySDK.shared.setManualConsent(false, for: .profiling);
            result(true);
            break;
        case "disableMonitoringConsent":
            ProximitySDK.shared.setManualConsent(false, for: .monitoring);
            result(true);
            break;
        case "disableAdvertisingConsent":
            ProximitySDK.shared.setManualConsent(false, for: .advertising);
            result(true);
            break;
        case "disableAdvancedConsent":
            ProximitySDK.shared.setManualConsent(false, for: .advancedTracking);
            result(true);
            break;
        case "checkPendingPermissions":
            break;
        case "requestTrackingAuthorization":
            ProximitySDK.shared.requestTrackingAuthorization()
            result(true);
            break;
        case "requestNotificationAuthorization":
            ProximitySDK.shared.requestNotificationAuthorization()
            result(true);
            break;
        case "requestLocationAuthorization":
            ProximitySDK.shared.requestLocationAuthorization()
            result(true);
            break;
        case "getInstallationId":
            result(ProximitySDK.shared.installationId())
            break;
        case "setAdvertisingEnabled":
            ProximitySDK.shared.advertisingEnabled=true;
            result(ProximitySDK.shared.advertisingEnabled)
            break;
        case "setAdvertisingDisabled":
            ProximitySDK.shared.advertisingEnabled=false;
            result(ProximitySDK.shared.advertisingEnabled)
            break;
        case "enabledCmp":
            ProximitySDK.shared.cmpEnabled.toggle();
            result(ProximitySDK.shared.cmpEnabled);
            break;
        case "setBaseJobId":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "setBaseJobId for iOS Not Implemented",
                                details: nil));
            break;
        case "getAdvertisingIdentifier":
            result(ProximitySDK.shared.advertisingId());
            break;
        case "getApiKey":
            result(ProximitySDK.shared.apiKey);
            break;
        case "getContext":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "getContext for iOS Not Implemented",
                                details: nil));
            break;
        case "getGDPRConsent":
            guard let args = call.arguments else {
                return
            }
            if let myArgs = args as? [String: Any],
                let vendor = myArgs["VENDOR_ID"] as? Int32{
                result(ProximitySDK.shared.getGDPRConsent(forVendor: vendor));
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                   "flutter arguments in method: (getGDPRConsent)", details: nil))
            }
            break;
        case "getSecret":
            result(ProximitySDK.shared.apiSecret);
            break;
        case "getSessionId":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "getSessionId for iOS Not Implemented",
                                details: nil));
            break;
        case "isLimitAdTrackingEnabled":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "isLimitAdTrackingEnabled for iOS Not Implemented",
                                details: nil));
            break;
        //da fare
        case "getCustomActionListeners":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "getCustomActionListeners for iOS Not Implemented",
                                details: nil));
            break;
        case "addCustomActionListener":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "addCustomActionListener for iOS Not Implemented",
                                details: nil));
            break;
        case "removeCustomActionListener":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "removeCustomActionListener for iOS Not Implemented",
                                details: nil));
            break;

        case "getAdvertisingEnabled":
            result(ProximitySDK.shared.advertisingEnabled)
            break;
        case "setGDPRConsent":
            guard let args = call.arguments else {
                return
            }
            if let myArgs = args as? [String: Any],
                let vendor = myArgs["vendor"] as? Int32,
                let consent = myArgs["consent"] as? Bool{
                ProximitySDK.shared.setGDPRConsent(consent, forVendor: vendor)
                result(ProximitySDK.shared.getGDPRConsent(forVendor: vendor));
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                   "flutter arguments in method: (setGDPRConsent)", details: nil))
            }
            break;
        case "setInstallationId":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "setInstallationId for iOS Not Implemented",
                                details: nil));
            break;
        case "setSessionId":
            result(FlutterError(code: "UNAVAILABLE",
                                message: "setSessionId for iOS Not Implemented",
                                details: nil));
            break;
        default:
            result(FlutterError(code: "UNAVAILABLE",
                                message: "Unknown Method for iOS Not Implemented",
                                details: nil));
            break;
    }
  }
    class SwiftStreamHandler: NSObject, FlutterStreamHandler, UIApplicationDelegate, JTProximityCustomDelegate {
        public var eventSink: FlutterEventSink?
        public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
            eventSink = events
            /*
            events(true) // any generic type or more compex dictionary of [String:Any]
            events(FlutterError(code: "ERROR_CODE",
                                 message: "Detailed message",
                                 details: nil)) // in case of errors
            events(FlutterEndOfEventStream) // when stream is over
            */
            return nil
        }

        public func onCancel(withArguments arguments: Any?) -> FlutterError? {
            eventSink = nil
            return nil
        }
        
        public func jtProximityDidReceiveCustomAction(_ customAction: String?) {
            if eventSink != nil {
                eventSink!(customAction)
            }
        }
    }
}
